// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       indexed_character.h
 *  \brief      インデックス化した文字の各種宣言
 *  \date       Since: June 28, 2021. 9:33:33 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_INDEXED_CHARACTER_H_1624840413
#define INCGUARD_INDEXED_CHARACTER_H_1624840413

#include <cstdint>

namespace msgconv
{
//! インデックス化文字
using IndexedCharacter = uint16_t;

constexpr IndexedCharacter kIndexedCharacterReserveStart                    = 0xE000;   //!< 予約開始インデックス
constexpr IndexedCharacter kIndexedCharacterSubstituteOptionReserveStart    = 0xE000;   //!< 置換フォーマットオプションの開始インデックス
constexpr IndexedCharacter kIndexedCharacterSubstituteOptionReserveEnd      = 0xEFFF;   //!< 置換フォーマットオプションの終了インデックス
constexpr IndexedCharacter kIndexedCharacterNewLine                         = 0xFFF0;   //!< 改行
constexpr IndexedCharacter kIndexedCharacterSubstitute                      = 0xFFF1;   //!< 置換
constexpr IndexedCharacter kIndexedCharacterInvalid                         = 0xFFFE;   //!< 無効なインデックス
constexpr IndexedCharacter kIndexedCharacterEndOfText                       = 0xFFFF;   //!< テキスト終了

// 置換フォーマットのマスク値
constexpr IndexedCharacter kSubstituteFormatMaskIndex           = 0xE000;   //!< 置換フォーマットのインデックスのマスク値
constexpr IndexedCharacter kSubstituteFormatMaskWidth           = 0xE100;   //!< 置換フォーマットの桁幅のマスク値
constexpr IndexedCharacter kSubstituteFormatMaskPrecision       = 0xE200;   //!< 置換フォーマットの小数点以下の桁幅のマスク値
constexpr IndexedCharacter kSubstituteFormatMaskFillCharacter   = 0xE300;   //!< 置換フォーマットの桁幅埋め文字のマスク値
constexpr IndexedCharacter kSubstituteFormatMaskHexWidth        = 0xE400;   //!< 置換フォーマットの16進数の桁幅のマスク値
constexpr IndexedCharacter kSubstituteFormatMaskFlags           = 0xE500;   //!< 置換フォーマットのフラグのマスク値

// 置換フォーマットのフラグオプションのビット構成
constexpr uint8_t kSubstituteFlagBitShowPlusSign       = 0;     //!< kSubstituteFormatMaskFlags内の符号表示フラグのビット位置
constexpr uint8_t kSubstituteFlagBitShowSeparator      = 1;     //!< kSubstituteFormatMaskFlags内の区切り文字表示フラグのビット位置
constexpr uint8_t kSubstituteFlagBitSkipAutoIndex      = 2;     //!< kSubstituteFormatMaskFlags内の自動インデックススキップフラグのビット位置
constexpr uint8_t kSubstituteFlagBitValueDisplayModeLo = 3;     //!< kSubstituteFormatMaskFlags内の値の表示モードのビット位置（2ビットのうちの下位）
constexpr uint8_t kSubstituteFlagBitValueDisplayModeHi = 4;     //!< kSubstituteFormatMaskFlags内の値の表示モードのビット位置（2ビットのうちの上位）

}   // namespace msgconv

#endif	// INCGUARD_INDEXED_CHARACTER_H_1624840413


// vim: et ts=4 sw=4 sts=4
