// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       msgconv.cc
 *  \brief      メッセージコンバータ
 *  \date       Since: October 20, 2018. 16:16:15 JST.
 *  \date       Last Change: October 20, 2018. 23:50:53 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <mtk/text_format.h>

#include "binary_generator.h"
#include "message.h"
#include "run_options.h"
#include "source_generator.h"

constexpr const char *kProgramName = "mgl-msgconv";
constexpr const char *kVersionInfo = "1.2.0";

/* ------------------------------------------------------------------------- */
/*!
 *  \brief      バージョン情報の表示
 */
/* ------------------------------------------------------------------------- */
void PrintVersionInfo() noexcept
{
    printf("%s version %s (compiled at %s %s)\n", kProgramName, kVersionInfo, __DATE__, __TIME__);
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      メイン
 *  \param[in]  argc    コマンドライン引数の数
 *  \param[in]  argv    コマンドライン引数
 *  \return     実行結果
 */
/* ------------------------------------------------------------------------- */
int main(int argc, const char *argv[])
{
    // メッセージを初期化
    msgconv::InitMessage();

    // コマンドライン引数を解釈
    msgconv::RunOptions runOptions;
    if (!runOptions.ParseCommandlineArgs(argc, argv))
    {
        return EXIT_FAILURE;
    }

    // バージョン情報の表示
    if (runOptions.ShouldPrintVersion())
    {
        PrintVersionInfo();
        return EXIT_SUCCESS;
    }
    // 簡易ヘルプの表示
    else if (runOptions.ShouldPrintHelp())
    {
        PrintVersionInfo();
        msgconv::PutMessage(msgconv::MsgID::HelpText);
        return EXIT_SUCCESS;
    }

    const auto &options = runOptions.GetOptions();

    // ソースコード出力
    if (options.generateSource)
    {
        // 警告表示
        if (!options.fontMetricPath.empty())
        {
            msgconv::PutWarning(msgconv::MsgID::NotSupportedIndededString);
        }

        msgconv::SourceGenerator generator;
        if (!generator.Generate(options))
        {
            return EXIT_FAILURE;
        }
    }
    // バイナリ出力
    else
    {
        msgconv::BinaryGenerator generator;
        if (!generator.Generate(options))
        {
            return EXIT_FAILURE;
        }
    }
    msgconv::PutMessage(msgconv::MsgID::Succeeded);

    return EXIT_SUCCESS;
}

// vim: et ts=4 sw=4 sts=4

