// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       environment.cc
 *  \brief      動作環境に関わる定義・実装
 *  \date       Since: January 23, 2023. 16:00:04 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mtk/environment.h>

#include <stdlib.h>

namespace MTK::Environment
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      環境変数の取得
 *  \param[in]  name    変数名
 *  \return     変数名に対応した値．見つからない場合は空の文字列
 */
/* ------------------------------------------------------------------------- */
std::string GetValue(const char *name) noexcept
{
#if defined(MTK_TARGET_WINDOWS)
    char *value;
    size_t length;
    _dupenv_s(&value, &length, name);
    if ((value != nullptr) && (length >= 1))
    {
        std::string result(value);
        free(value);
        return result;
    }
#else
    if (const auto *value = getenv(name); value != nullptr)
    {
        return value;
    }
#endif

    return "";
}
std::filesystem::path GetHomeDirectory() noexcept
{
#if defined(MTK_TARGET_WINDOWS)
    std::filesystem::path path = MTK::Environment::GetValue("HOMEDRIVE") + MTK::Environment::GetValue("HOMEPATH");
#else
    std::filesystem::path path = MTK::Environment::GetValue("HOME");
#endif

    return path;
}
}   // namespace MTK::Environment

// vim: et ts=4 sw=4 sts=4
