// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       file_pointer.cc
 *  \brief      ファイルポインタクラス
 *  \date       Since: January 23, 2023. 16:12:27 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#include <mtk/file_pointer.h>

#include <mtk/utility.h>

namespace MTK::File
{
/* ------------------------------------------------------------------------- */
/*!
 *  \brief      コンストラクタ
 *  \param[in]  path    ファイルのパス
 *  \param[in]  mode    オープンモード
 */
/* ------------------------------------------------------------------------- */
Pointer::Pointer(const std::filesystem::path &path, const char *mode) noexcept
    : _fp(nullptr)
{
    Open(path, mode);
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      デストラクタ
 */
/* ------------------------------------------------------------------------- */
Pointer::~Pointer() noexcept
{
    Close();
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルのオープン
 *  \param[in]  path    ファイルのパス
 *  \param[in]  mode    オープンモード
 *  \retval     true    成功
 *  \retval     false   失敗
 */
/* ------------------------------------------------------------------------- */
bool Pointer::Open(const std::filesystem::path &path, const char *mode) noexcept
{
    Close();

#if defined(MTK_TARGET_WINDOWS)
    _wfopen_s(&_fp, path.c_str(), Utility::ToWindowsWideChar(mode).c_str());
#else
    _fp = fopen(path.c_str(), mode);
#endif

    return IsOpened();
}


/* ------------------------------------------------------------------------- */
/*!
 *  \brief      ファイルのクローズ
 */
/* ------------------------------------------------------------------------- */
void Pointer::Close() noexcept
{
    if (IsOpened())
    {
        fclose(_fp);
        _fp = nullptr;
    }
}

}   // namespace MTK::File

// vim: et ts=4 sw=4 sts=4
