// SPDX-License-Identifier: Zlib
/* ------------------------------------------------------------------------- */
/*!
 *  \file       source_generator.h
 *  \brief      ソースコード埋め込み用データの出力クラス
 *  \date       Since: October 20, 2018. 16:47:32 JST.
 *  \date       Last Change: October 20, 2018. 17:56:47 JST.
 *  \author     Acerola
 */
/* ------------------------------------------------------------------------- */

#ifndef INCGUARD_SOURCE_GENERATOR_H_1540021652
#define INCGUARD_SOURCE_GENERATOR_H_1540021652

#include <mtk/csv_parser.h>

#include "run_options.h"
#include "text_replacer.h"

namespace msgconv
{
//! ソースコード埋め込み用データの出力クラス
class SourceGenerator : public TextReplaceCommandProcessor
{
public:
    SourceGenerator() noexcept;

    bool Generate(const RunOptions::Options &options) noexcept;

private:
    bool GenerateIDList(const MTK::CSVParser &csvParser, const std::filesystem::path &outputPrefix) const noexcept;
    bool GenerateMessageList(const MTK::CSVParser &csvParser, const std::filesystem::path &outputPrefix, int fieldIndex) const noexcept;

    bool PutIDList(FILE *fp, const MTK::CSVParser &csvParser, const std::string &indent = "") const noexcept;
    bool PutMessageList(FILE *fp, const MTK::CSVParser &csvParser, int fieldIndex, const std::string &indent = "") const noexcept;

    virtual bool DoReplaceCommand(FILE *fp, const TextReplacer::Node &node, const std::filesystem::path &srcFilePath) noexcept override;

    MTK::CSVParser _csvParser;
};
}   // namespace msgconv

#endif  // INCGUARD_SOURCE_GENERATOR_H_1540021652

// vim: et ts=4 sw=4 sts=4
